from .secretsmngr import SecretsManager
from .lambdaexceptions import *
from .lambdahandler import *
from .defaultvars import *
from .dynamodb import *
from .realtimesecretsmngrconfigprovider import *
from .secretsmngrconfigprovider import *
from .s3configprovider import *
from .realtimes3configprovider import *
from .scheduleds3configprovider import *
from .scheduledsecretsmngrconfigprovider import *
from .dynamodbconfigprovider import *
from .realtimedynamodbconfigprovider import *


import boto3
from appsyncclient import AppSyncClient
import os
import json

import logging, base64

from pspring import Configuration

logger = logging.getLogger(__name__)

config = Configuration.getConfig(__name__)

class RealTimeS3():
    def __init__(self,**kargs):
        self.region = kargs.get("region")
        self.bucketId = kargs.get("bucketId") or config.getProperty("bucketId")
        self.objectKey = kargs.get("objectKey") or config.getProperty("objectKey")
        self.apiId = kargs.get("apiId") or config.getProperty("apiId")

        if(self.bucketId == None or self.objectKey == None or self.region == None):
            logger.error("region,bucketId and objectKey required")
            raise Exception("configuration error")
        self.client = AppSyncClient(region=self.region,apiId=self.apiId)

    def getValue(self):
        client = boto3.client("s3",region_name=self.region)
        file = client.get_object(Bucket=self.bucketId,Key=self.objectKey)
        filecontent = file.get("Body").read().decode("utf-8")
        
        return str(filecontent)

    def subscribe(self,callback):
        contents = self.getValue()
        callback(contents)
        id = "arn:aws:s3:::"+self.bucketId+":"+self.objectKey
        query = json.dumps({"query": "subscription {\n  updatedResource(id:\""+id+"\") {\n    id\n    data\n  }\n}\n"})

        def secretcallback(client, userdata, msg):
            logger.debug("New data received : "+str(msg))

            callbackdatab64 = json.loads(msg.payload).get("data",{}).get("updatedResource",{}).get("data")
            logger.debug(callbackdatab64)
            try:
                callbackdata = base64.b64decode(callbackdatab64.encode("utf-8"))
                logger.debug(f"decoded successfully {callbackdata}")
                callback(callbackdata.decode("utf-8"))
            except Exception as e:
                logger.error(str(e))
                
        response = self.client.execute(data=query,callback=secretcallback)



from pspring import ConfigurationProvider, Configuration
from .secretsmngr import SecretsManager
import logging,json
from datetime import datetime

logger = logging.getLogger(__name__)
config = Configuration.getConfig(__name__)

class SecretsMgrConfigProvider(ConfigurationProvider):
    def __init__(self,**kargs):
        secretId = kargs.get("secretId") or config.getProperty("secretId")
        region = kargs.get("region") or config.getProperty("region")
        self.subscriptions = []
        self.mgr = SecretsManager(secretId=secretId,region=region)
        self.refresh()
        self.lastUpdated = int(datetime.now().timestamp())
        self.timeout = kargs.get("timeout") or config.getProperty("PSPRING_AWS_SM_CONFIG_TIMEOUT") or config.getProperty("PSPRING_CONFIG_TIMEOUT")

    def getProperty(self,propertyName):
        self.checkForRefresh()
        return self.secretValue.get(propertyName)

    def checkForRefresh(self):
        lastUpdatedSeconds = int(datetime.now().timestamp()) - self.lastUpdated
        if(self.timeout != None and lastUpdatedSeconds > int(self.timeout)):
            self.lastUpdated = int(datetime.now().timestamp())
            self.refresh()

    def refresh(self):
        self.secretValue = self.mgr.getSecretValue()
        self.publish()

    def subscribe(self,callback):
        self.subscriptions.append(callback)

    def publish(self):
        for subscription in self.subscriptions:
            subscription()


from pspring import ConfigurationProvider, Configuration
import logging,json

from .dynamodb import DynamoDBTable

import boto3

logger = logging.getLogger(__name__)
config = Configuration.getConfig(__name__)

class DynamodbConfigProvider(ConfigurationProvider):
    def __init__(self,**kargs):

        self.tableName = kargs.get("tableName") or config.getProperty("tableName")
        self.primaryKey = kargs.get("primaryKey") or config.getProperty("primaryKey")
        self.primaryKeyName = kargs.get("primaryKeyName") or config.getProperty("primaryKeyName")
        self.sortKey = kargs.get("sortKey") or config.getProperty("sortKey")
        self.sortKeyName = kargs.get("sortKeyName") or config.getProperty("sortKeyName")
        self.region = kargs.get("region") or config.getProperty("region")
        self.configColumnName = kargs.get("configColumnName") or config.getProperty("configColumnName")

        @DynamoDBTable(
            tableName=self.tableName,
            primaryKey=self.primaryKeyName,
            sortKey=self.sortKeyName,
            region=self.region
        )
        class DynamoTable():
            pass
        self.table = DynamoTable()
        self.config = {}
        self.subscriptions = []
        self.refresh()

    def getProperty(self,propertyName):
        return self.config.get(propertyName)

    def refresh(self):
        dynamoContent = self.table.get(self.primaryKey,sortKey=self.sortKey,column=self.configColumnName)
        try:
            self.config = json.loads(str(dynamoContent))
            self.publish()
        except Exception as er:
            logger.warn(f"Received content from dynamodb was not json {er}")

    def subscribe(self,callback):
        self.subscriptions.append(callback)

    def publish(self):
        for subscription in self.subscriptions:
            subscription()


from pspring import ConfigurationProvider, Configuration
from .realtimes3 import RealTimeS3
import logging
import json

logger = logging.getLogger(__name__)
config = Configuration.getConfig(__name__)

class RealTimeS3ConfigProvider(ConfigurationProvider):
    def __init__(self,**kargs):
        self.subscriptions = []
        region = kargs.get("region") or config.getProperty("region")
        apiId = kargs.get("apiId") or config.getProperty("apiId")
        bucketId = kargs.get("bucketId") or config.getProperty("bucketId")
        objectKey = kargs.get("objectKey") or config.getProperty("objectKey")

        self.s3client = RealTimeS3(bucketId=bucketId,objectKey=objectKey,apiId=apiId,region=region)
        self.s3client.subscribe(self.eventCallBack)

    def eventCallBack(self,response):
        logger.info("Received updated data "+str(response))
        self.response = json.loads(response)
        logger.info("After json")
        self.publish()

    def refresh(self):
        pass

    def publish(self):
        for subscription in self.subscriptions:
            subscription()

    def getProperty(self,propertyName):
        return self.response.get(propertyName)

    def subscribe(self,callback):
        self.subscriptions.append(callback)



import boto3
from appsyncclient import AppSyncClient
import os
import json

import logging
import base64

from pspring import Configuration
from .dynamodb import DynamoDBTable

logger = logging.getLogger(__name__)

config = Configuration.getConfig(__name__)

class RealTimeDynamoDB():
    def __init__(self,**kargs):
        self.tableName = kargs.get("tableName") or config.getProperty("tableName")
        self.primaryKey = kargs.get("primaryKey") or config.getProperty("primaryKey")
        self.primaryKeyName = kargs.get("primaryKeyName") or config.getProperty("primaryKeyName")
        self.sortKey = kargs.get("sortKey") or config.getProperty("sortKey")
        self.sortKeyName = kargs.get("sortKeyName") or config.getProperty("sortKeyName")
        self.region = kargs.get("region") or config.getProperty("region")
        self.configColumnName = kargs.get("configColumnName") or config.getProperty("configColumnName")

        self.tableAsConfig = kargs.get("tableAsConfig") or config.getProperty("tableAsConfig") or "False"

        if self.tableAsConfig == "False":
            @DynamoDBTable(
                tableName=self.tableName,
                primaryKey=self.primaryKeyName,
                sortKey=self.sortKeyName,
                region=self.region
            )
            class DynamoTable():
                pass
            self.table = DynamoTable()

        self.config = {}

        self.apiId = kargs.get("apiId") or config.getProperty("apiId")

        if(self.tableName == None or (self.tableAsConfig == "False" and (self.primaryKey == None or self.sortKey == None))):
            logger.error("mandatory fields missing")
            raise Exception("configuration error")
        self.client = AppSyncClient(region=self.region,apiId=self.apiId)

    def getValue(self):
        dynamoContent = self.table.get(self.primaryKey,scope=self.sortKey,column=self.configColumnName)
        return str(dynamoContent)

    def subscribe(self,callback):
        if self.tableAsConfig == "False":
            contents = self.getValue()
            callback(contents)
        id = "arn:aws:dynamodb:::"+self.tableName+":"+self.primaryKey
        if self.sortKey != None:
            id = id + ":"+self.sortKey

        if self.tableAsConfig == "True":
            id = "arn:aws:dynamodb:::"+self.tableName

        query = json.dumps({"query": "subscription {\n  updatedResource(id:\""+id+"\") {\n    id\n    data\n  }\n}\n"})

        def secretcallback(client, userdata, msg):
            logger.debug("New data received : "+str(msg))
            callbackdatab64 = json.loads(msg.payload).get("data",{}).get("updatedResource",{}).get("data")
            logger.debug(callbackdatab64)
            if ( self.configColumnName != None ):
                try:
                    callbackdata = base64.b64decode(callbackdatab64.encode())
                    logger.debug(f"decoded successfully {callbackdata}")
                    if self.tableAsConfig == "False":
                        callbackdatacolumn = json.loads(callbackdata.decode()).get(self.configColumnName)
                    else:
                        callbackdatacolumn = json.loads(callbackdata.decode())
                except Exception as e:
                    logger.error(str(e))
                

            callback(callbackdatacolumn)

        response = self.client.execute(data=query,callback=secretcallback)



from pspring import ConfigurationProvider, Configuration
from .realtimes3 import RealTimeS3
import logging
import json
from .dynamodb import DynamoDBTable
from .realtimedynamodb import RealTimeDynamoDB

logger = logging.getLogger(__name__)
config = Configuration.getConfig(__name__)

class RealTimeDynamodbConfigProvider(ConfigurationProvider):
    def __init__(self,**kargs):
        self.subscriptions = []
        self.tableName = kargs.get("tableName") or config.getProperty("tableName")
        self.primaryKey = kargs.get("primaryKey") or config.getProperty("primaryKey")
        self.primaryKeyName = kargs.get("primaryKeyName") or config.getProperty("primaryKeyName")
        self.sortKey = kargs.get("sortKey") or config.getProperty("sortKey")
        self.sortKeyName = kargs.get("sortKeyName") or config.getProperty("sortKeyName")
        self.region = kargs.get("region") or config.getProperty("region")
        self.configColumnName = kargs.get("configColumnName") or config.getProperty("configColumnName")
        self.apiId = kargs.get("apiId") or config.getProperty("apiId")
        self.tableAsConfig = kargs.get("tableAsConfig") or config.getProperty("tableAsConfig")

        self.dynamodbclient = RealTimeDynamoDB(**kargs)
        self.dynamodbclient.subscribe(self.eventCallBack)
        self.response = {}

    def eventCallBack(self,response):
        #response = response.replace("\"","\\\"")
        logger.info("Received updated data "+str(response))
        
        if self.tableAsConfig == "True":
            try:
                self.response.update({
                    response.get(self.primaryKeyName) : {
                        response.get(self.sortKeyName) : response
                    }
                })
            except Exception as e:
                logger.error(str(e))
        else:
            try:
                self.response = response
            except Exception as e:
                logger.error(str(e))

        logger.info(self.response)
        self.publish()

    def refresh(self):
        pass

    def publish(self):
        for subscription in self.subscriptions:
            subscription()

    def getProperty(self,propertyName):
        return self.response.get(propertyName)

    def subscribe(self,callback):
        self.subscriptions.append(callback)



from pspring import ConfigurationProvider, Configuration
from realtimeawssecretsmngr import RealTimeAwsSecretsMngr
import logging
import json

logger = logging.getLogger(__name__)
config = Configuration.getConfig(__name__)

class RealTimeSecretsMgrConfigProvider(ConfigurationProvider):
    def __init__(self,**kargs):
        self.subscriptions = []
        secretId = kargs.get("secretId") or config.getProperty("secretId")
        region = kargs.get("region") or config.getProperty("region")
        apiId = kargs.get("apiId") or config.getProperty("apiId")

        self.mgr = RealTimeAwsSecretsMngr(secretId=secretId,apiId=apiId,region=region)
        self.mgr.subscribe(self.eventCallBack)

    def eventCallBack(self,response):
        logger.info("Received updated secret "+str(response))
        self.response = json.loads(response)
        self.publish()

    def refresh(self):
        pass

    def publish(self):
        for subscription in self.subscriptions:
            subscription()

    def getProperty(self,propertyName):
        return self.response.get(propertyName)

    def subscribe(self,callback):
        self.subscriptions.append(callback)



import boto3
from appsyncclient import AppSyncClient
import os
import json

import logging, threading

from pspring import Configuration, ConfigurationProvider

logger = logging.getLogger(__name__)

config = Configuration.getConfig(__name__)

class ScheduledSecretsMngrConfigProvider(ConfigurationProvider):
    def __init__(self,**kargs):
        self.region = kargs.get("region") or config.getProperty("region")
        self.secretId = kargs.get("secretId") or config.getProperty("secretId")
        self.period = kargs.get("period") or config.getProperty("period")
        self.period = int(self.period)
        if(self.secretId == None or self.region == None or self.period == None):
            logger.error("region,bucketId and objectKey required")
            raise Exception("configuration error")

        self.response = {}
        self.subscriptions=[]
        self.client = boto3.client('secretsmanager',region_name=self.region)
        
        self.loop()

    def loop(self):
        self.refresh()

        thread = threading.Timer(self.period,self.loop)
        thread.daemon = True
        thread.start()
    
    def refresh(self):
        logger.info("Getting value")
        
        secretResponse = self.client.get_secret_value(SecretId=self.secretId)
        content = secretResponse.get("SecretString")
        
        try:
            self.response = json.loads(content)
        except Exception as jsonerror:
            logger.error("Error while loading json..")
        self.publish()

    def publish(self):
        for subscription in self.subscriptions:
            subscription()

    def getProperty(self,propertyName):
        return self.response.get(propertyName)

    def subscribe(self,callback):
        self.subscriptions.append(callback)


import boto3
from appsyncclient import AppSyncClient
import os
import json

import logging, threading

from pspring import Configuration, ConfigurationProvider

logger = logging.getLogger(__name__)

config = Configuration.getConfig(__name__)

class ScheduledS3ConfigProvider(ConfigurationProvider):
    def __init__(self,**kargs):
        self.region = kargs.get("region") or config.getProperty("region")
        self.bucketId = kargs.get("bucketId") or config.getProperty("bucketId")
        self.objectKey = kargs.get("objectKey") or config.getProperty("objectKey")
        self.period = kargs.get("period") or config.getProperty("period")
        self.period = int(self.period)
        if(self.bucketId == None or self.objectKey == None or self.region == None or self.period == None):
            logger.error("region,bucketId and objectKey required")
            raise Exception("configuration error")

        self.response = {}
        self.subscriptions=[]
        
        self.loop()

    def loop(self):
        self.refresh()
        thread = threading.Timer(self.period,self.loop)
        thread.daemon = True
        thread.start()
    
    def refresh(self):
        logger.info("Getting value")
        client = boto3.client("s3",region_name=self.region)
        file = client.get_object(Bucket=self.bucketId,Key=self.objectKey)
        filecontent = file.get("Body").read().decode("utf-8")
        
        try:
            self.response = json.loads(filecontent)
        except Exception as jsonerror:
            logger.error("Error while loading json..")
        self.publish()

    def publish(self):
        for subscription in self.subscriptions:
            subscription()

    def getProperty(self,propertyName):
        return self.response.get(propertyName)

    def subscribe(self,callback):
        self.subscriptions.append(callback)


from .lambdaexceptions import *
import json
import logging, traceback

logger = logging.getLogger("pspring-aws")

class LambdaHandler():

    def __init__(self,*args,**kargs):
        self.type = kargs.get("type","proxy")
        self.corsHeaders = kargs.get("corsheaders")

    def __call__(self,classObj):
        handlerOrig = classObj.handler
        constructorOrig = classObj.__init__
        def constructor(*args,**kargs):
            selfObj = args[0]
            selfObj.logattrs = {}
            constructorOrig(*args,**kargs)

        def logtemplate(selfObj,logObj):
            logdict=dict()
            if type(logObj) == type(""):
                logdict.update(selfObj.logattrs)
                logdict.update({
                    "message" : logObj
                })
                return logdict
            else:
                logdict.update(selfObj.logattrs)
                logdict.update(logObj)
                return logdict

        def debug(selfObj,logObj):
            logger.debug(selfObj.logtemplate(logObj))
        def info(selfObj,logObj):
            logger.info(selfObj.logtemplate(logObj))
        def error(selfObj,logObj):
            logger.error(selfObj.logtemplate(logObj))
        def warn(selfObj,logObj):
            logger.warn(selfObj.logtemplate(logObj))

        def addToLogger(selfObj,fieldName,fieldValue):
            selfObj.logattrs.update({
                fieldName : fieldValue
            })

        def handler(*args,**kargs):
            finalresponse = None
            try:
                selfObj = args[0]
                event = args[1]
                context = args[2]
                econtext = event.get('requestContext',{})
                requestId = str(econtext.get('requestId',{}))
                traceId = event.get("headers",{}).get("X-Amzn-Trace-Id")
                cfId = event.get("headers",{}).get("X-Amz-Cf-Id")
                selfObj.addToLogger("requestId",requestId)

                if hasattr(context,"aws_request_id"):
                    selfObj.addToLogger("awsRequestId",context.aws_request_id)

                if traceId != None:
                    selfObj.addToLogger("X-Amzn-Trace-Id",traceId)

                if cfId != None:
                    selfObj.addToLogger("X-Amz-Cf-Id",cfId)

                if str(econtext.get('requestId')) == "COLD_START_WARMER":
                    selfObj.debug("Received cold start request")
                    return {
                        "statusCode" : "200",
                        "body" : json.dumps("{}")
                    }

                response = handlerOrig(*args,**kargs)
                finalresponse = {
                    "statusCode" : "200"
                }
                finalresponse.update(response)
                
                return finalresponse

            except LambdaException as le:
                 finalresponse = le.getResponse()
            except Exception as e:
                logger.error(traceback.format_exc()+str(e))
                if hasattr(e,"statusCode") and hasattr(e,"response"):
                    finalresponse =  LambdaException(str(e),e.statusCode,e.response).getResponse()
                else:
                    finalresponse =  LambdaException(str(e)).getResponse()

            if(self.corsHeaders != None and type(self.corsHeaders) == type({})):
                headers = finalresponse.get("headers",{})
                headers.update(self.corsHeaders)
                finalresponse.update({"headers" : headers})
            return finalresponse

        classObj.handler = handler
        classObj.__init__ = constructor
        classObj.addToLogger = addToLogger
        classObj.debug = debug
        classObj.info = info
        classObj.warn = warn
        classObj.error = error
        classObj.logtemplate = logtemplate
        return classObj



import boto3
import time

from pspring import Configuration

import logging

logger = logging.getLogger(__name__)
config = Configuration.getConfig(__name__)

class DynamoDBTable():
    def __init__(self,*args,**kargs):
        self.tableName = kargs.get("tableName")
        self.primaryKey = kargs.get("primaryKey")
        self.sortKey = kargs.get("sortKey")
        self.region = kargs.get("region") or config.getProperty("region")
        self.ttlcolumnname = kargs.get("ttlColumnName") or config.getProperty("ttlColumnName")
        self.ttl = int(kargs.get("ttl") or config.getProperty("ttl"))
        self.init()

    def init(self):
        dynamodb = boto3.resource("dynamodb",region_name=self.region)
        self.table = dynamodb.Table(self.tableName)


    def __call__(self,classObj):
        def put(selfObj,data):
            if(self.ttlcolumnname != None and data.get(self.ttlcolumnname) == None):
                data[self.ttlcolumnname] = int(time.time())+self.ttl
            self.table.put_item(Item=data)

        def update(selfObj,data):
            key = {}
            updateExpressionItems = {}
            expressionAttributeValues = {}
            key[self.primaryKey] = data.get(self.primaryKey)
            if self.sortKey != None:
                key[self.sortKey] = data.get(self.sortKey)

            for attribute in data:
                if attribute != self.primaryKey and attribute != self.sortKey:
                    updateExpressionItems[attribute]=f":{attribute}"
                    expressionAttributeValues[f":{attribute}"] = attribute

            updateExpression = "SET "
            firstItem = True
            for item in updateExpressionItems.items():
                if firstItem == True:
                    updateExpression = f"{updateExpression} {item[0]} = {item[1]}"
                else:
                    updateExpression = f"{updateExpression}, {item[0]} = {item[1]}"

            return selfObj.__update__(key,updateExpression,expressionAttributeValues)

        def __update__(selfObj,key,updateExpression,expressionAttributeValues):
            return self.table.update_item(Key=key,UpdateExpression = updateExpression,ExpressionAttributeValues=expressionAttributeValues,ReturnValues="UPDATED_NEW")

        def get(selfObj,primaryKey,**kargs):
            key = {}
            key[self.primaryKey] = primaryKey
            sortKey = kargs.get(self.sortKey,None)
            columns = kargs.get("columns",None)
            column = kargs.get("column",None)
            if self.sortKey != None and sortKey != None:
                key.update({
                    self.sortKey : sortKey
                })
            response = self.table.get_item(Key=key)
            responseData = {}
            item = response.get("Item")
            if item != None:
                if columns != None:
                    for column in columns:
                        responseData.update({
                            column : item.get(column)
                        })
                    return responseData
                if column != None:
                    return item.get(column)
            return item
        classObj.put = put
        classObj.get = get
        classObj.update = update
        return classObj



import json

class LambdaException(Exception):
    def __init__(self,*args,**kargs):
        super().__init__(*args,**kargs)
        self.code = "500"
        self.msg = "Internal Server Error : "+str(self)
        self.payload = None
        if len(args) > 1:
            self.code = args[1]
        if len(args) > 0:
            self.msg = args[0]
        if len(args) > 2:
            self.payload = args[2]

        self.errorCode = kargs.get("errorCode")

    def getResponse(self):

        if self.payload != None:
            return {
                "statusCode": self.code,
                "body": self.payload
            }
        return {
            "statusCode": self.code,
            "body": json.dumps({
                "message" : self.msg,
                "code" : self.errorCode
            })
        }

class NotFoundException(LambdaException):
    def __init__(self,*args,**kargs):
        super().__init__(*args,**kargs)
        self.code="404"

class UnAuthorizedException(LambdaException):
    def __init__(self,*args,**kargs):
        super().__init__(*args,**kargs)
        self.code="401"

class Forbidden(LambdaException):
    def __init__(self,*args,**kargs):
        super().__init__(*args,**kargs)
        self.code="403"

class InternalServerException(LambdaException):
    def __init__(self,*args,**kargs):
        super().__init__(*args,**kargs)
        self.code="500"

class CreatedException(LambdaException):
    def __init__(self,*args,**kargs):
        super().__init__(*args,**kargs)
        self.code="201"

class MethodNotAllowed(LambdaException):
    def __init__(self,*args,**kargs):
        super().__init__(*args,**kargs)
        self.code="405"

class BadRequest(LambdaException):
    def __init__(self,*args,**kargs):
        super().__init__(*args,**kargs)
        self.code="400"

class UnSupportedMediaType(LambdaException):
    def __init__(self,*args,**kargs):
        super().__init__(*args,**kargs)
        self.code="415"

class RedirectException(LambdaException):
    def __init__(self,*args,**kargs):
        super().__init__(*args,**kargs)
        self.code="301"




import os, logging, json
import boto3
from pspring import *

logger = logging.getLogger(__name__)
config = Configuration.getConfig(__name__)

class SecretsManager():
    def __init__(self,*args,**kargs):
        self.secretId = kargs.get("secretId") or config.getProperty("secretId")
        self.region = kargs.get("region") or config.getProperty("region")

        if(self.secretId == None or self.region == None):
            logger.error("secretId required")
            raise Exception("configuration error")

        logger.info("Getting secret : "+self.secretId)

        self.client = boto3.client('secretsmanager',region_name=self.region)
        self.secretResponse = self.getSecret()
        logger.info("Secret status : OK")

    def getSecret(self):
        return self.client.get_secret_value(SecretId=self.secretId)

    def getSecretValue(self):
        secretString = self.secretResponse.get("SecretString")
        return json.loads(secretString)


class SecretValue():
    def __init__(self,*args,**kargs):
        self.secretId = kargs.get("secretId") or config.getProperty("secretId")
        self.region = kargs.get("region") or config.getProperty("region")
        self.column = kargs.get("column") or config.getProperty("column")
        self.columns = kargs.get("columns") or config.getProperty("columns")

        if(self.secretId == None or self.region == None):
            logger.error("secretId required")
            raise Exception("configuration error")

        self.client = boto3.client('secretsmanager',region_name=self.region)

    def __call__(self,classObj):
        def getSecretValue(selfObj):
            logger.info("Getting secret : "+self.secretId)
            self.secretResponse = self.getSecret()
            response = self.client.get_secret_value(SecretId=self.secretId)
            logger.info("Secret status : OK")
            responseData = {}
            columns = self.columns
            column = self.column
            if response != None:
                if columns != None:
                    for column in columns:
                        responseData.update({
                            column : item.get(column)
                        })
                    return responseData
                if column != None:
                    return item.get(column)
            return response

        classObj.getSecretValue = getSecretValue
        return classObj



from pspring import ConfigurationProvider, Configuration
import logging,json
import yaml
import os
import boto3
from datetime import datetime

logger = logging.getLogger(__name__)
config = Configuration.getConfig(__name__)

class S3ConfigProvider(ConfigurationProvider):
    def __init__(self,**kargs):
        self.bucketId = kargs.get("bucketId") or config.getProperty("bucketId")
        self.objectKey = kargs.get("objectKey") or config.getProperty("objectKey")
        self.region = kargs.get("region") or config.getProperty("region")
        self.subscriptions = []
        self.config={}
        self.refresh()
        self.lastUpdated = int(datetime.now().timestamp())
        self.timeout = config.getProperty("timeout") or config.getProperty("PSPRING_AWS_S3_CONFIG_TIMEOUT") or config.getProperty("PSPRING_CONFIG_TIMEOUT")

    def checkForRefresh(self):
        lastUpdatedSeconds = int(datetime.now().timestamp()) - self.lastUpdated
        if(self.timeout != None and lastUpdatedSeconds > int(self.timeout)):
            self.lastUpdated = int(datetime.now().timestamp())
            self.refresh()

    def getProperty(self,propertyName):
        self.checkForRefresh()
        # Check for json based keys
        config = self.config
        propertyNames = propertyName.split(".")
        for propertyNameItem in propertyNames:
            if config != None:
                config = config.get(propertyNameItem)

        if config != None:
            return config

        # return default way of property access
        return self.config.get(propertyName)

    def refresh(self):
        client = boto3.client("s3",region_name=self.region)
        file = client.get_object(Bucket=self.bucketId,Key=self.objectKey)
        filecontent = file.get("Body").read().decode("utf-8")
        try:
            extn = os.path.splitext(self.objectKey)[1]

            if extn and extn == '.json':
                self.config = json.loads(str(filecontent))
                logger.info(f"Loaded S3 file s3://{self.bucketId}/{self.objectKey} : {self.config}")
            elif extn and extn in ['.yml', '.yaml']:
                self.config = yaml.safe_load(filecontent)
            else:
                logger.error('File type {} not supported.'.format(extn))
                raise Exception('File type {} not supported.'.format(extn))

            self.publish()
        except Exception as er:
            logger.warn(f"Received content from s3 was not json {er}")

    def subscribe(self,callback):
        self.subscriptions.append(callback)

    def publish(self):
        for subscription in self.subscriptions:
            subscription()